/*
 * pcreg_ex.c - Perl compatible Regular Expression functions
 * 
 * include LICENSE
 */
#include <stdio.h>
#include <string.h>

#include <msglog.h>
#include <strmem.h>
#include <pcreg_ex.h>


/*
 *** \brief Allocates memory for a new Pcregex object.
 *   pattern : A  string containing the regular expression to be compiled.
 *   options : flags for compile regex -see pcre.h
 *   nmatch  : number of expected match, if pattern extraction.
 */

Pcregex *pcregex_new( char *pattern, int options, int nmatch )
{
   Pcregex *re;

   re =  app_new0(Pcregex, 1);
   pcregex_construct( re, pattern, options, nmatch );
   app_class_overload_destroy( (AppClass *) re, pcregex_destroy );
   return re;
}

/** \brief Constructor for the Pcregex object. */

void pcregex_construct( Pcregex *re, char *pattern, int options, int nmatch )
{
   app_class_construct( (AppClass *) re );

   re->pattern = app_strdup(pattern);
   re->options = options;
   re->nmatch = nmatch;
   if ( nmatch ) {
      re->ovector = app_new0( int, 3 * nmatch);  /* should be a multiple of 3 */
      re->matchStr = array_strPtr_new( nmatch );
   }
}

/** \brief Destructor for the Pcregex object. */

void pcregex_destroy(void *re)
{
   Pcregex *this = (Pcregex *) re;

   if (re == NULL) {
      return;
   }
   app_free(this->ovector);
   app_free(this->pattern);
   if ( this->creg ) {
      pcre_free( this->creg );
   }
   array_destroy( this->matchStr);

   app_class_destroy( re );
}


pcre *pcregex_compile(Pcregex *re)
{
   const char *errptr;             /* for error message */
   int erroffset;            /* for error offset */
   pcre *creg;

   creg = pcre_compile( re->pattern, re->options, &errptr, &erroffset, re->tableptr );
   if (creg == NULL) {
      msg_fatal( "Compilation failed at offset %d (in regexp '%s') '%s':\n  %s",
		 erroffset, re->pattern, errptr );
   }
   re->creg = creg;
   return creg;
}

/*
 *  return < 0, if no match found
 *  return 0 if match found and no extraction requested.
 *  return 0 if match found and not enough room in ovector.
 *  return > 0 if match found and enough room in ovector.
 */
int pcregex_exec(Pcregex *re, char *subject )
{
   int ret;
   
   if ( re->creg == NULL ){
      pcregex_compile( re );
   }
   int length = (int) strlen(subject);

   ret = pcre_exec( re->creg, NULL, subject, length, 0, 0, re->ovector, 3 * re->nmatch);
   if ( ret < 0 ) {
      msg_dbg("pcre_exec returned %d code", ret); 
   } else if ( re->nmatch ) {
       /* match found */
      int i;
      int len;
      int nmatch = ret;
      if ( ret == 0 ) {
	 nmatch = re->nmatch;
      }
      array_tbl_delete_all( re->matchStr );
      for ( i = 0 ; i < nmatch ; i++ ) {
	 len = re->ovector[2 * i + 1] - re->ovector[2 * i];
         char *newbuf = app_new0( char, len + 1);
         strncpy( newbuf, subject + re->ovector[2 * i], len);
	 array_tbl_add( re->matchStr, newbuf );
      }
   }
   return ret;
}

/*
 * Execute regex for string from dlist_lookup.
 * Look if regex match the string
 * return 0 if regex match
 */

int pcregex_str_cmp( AppClass *d1, AppClass *d2 )
{
   Pcregex *list_re = (Pcregex *) d1;
   char *str =  (char *) d2;
   
   if ( pcregex_exec(list_re, str ) < 0 ) {
      return 1;
   }
   return 0;
}

